==> FK-2_8_9.tar.bz2: FK-2_8_9/src/STLParser.cxx <==
/****************************************************************************
 *
 *	Copyright (c) 1999-2009, Fine Kernel Project, All rights reserved.
 *
 *	Redistribution and use in source and binary forms,
 *	with or without modification, are permitted provided that the
 *	following conditions are met:
 *
 *		- Redistributions of source code must retain the above
 *			copyright notice, this list of conditions and the
 *			following disclaimer.
 *
 *		- Redistributions in binary form must reproduce the above
 *			copyright notice, this list of conditions and the
 *			following disclaimer in the documentation and/or
 *			other materials provided with the distribution.
 *
 *		- Neither the name of the copyright holders nor the names
 *			of its contributors may be used to endorse or promote
 *			products derived from this software without specific
 *			prior written permission.
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *	"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *	LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *	FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *	COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *	HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *	STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 *	IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *	POSSIBILITY OF SUCH DAMAGE. 
 *
 ****************************************************************************/
/****************************************************************************
 *
 *	Copyright (c) 1999-2009, Fine Kernel Project, All rights reserved.
 *
 *	本ソフトウェアおよびソースコードのライセンスは、基本的に
 *	「修正 BSD ライセンス」に従います。以下にその詳細を記します。
 *
 *	ソースコード形式かバイナリ形式か、変更するかしないかを問わず、
 *	以下の条件を満たす場合に限り、再頒布および使用が許可されます。
 *
 *	- ソースコードを再頒布する場合、上記の著作権表示、本条件一覧、
 *		および下記免責条項を含めること。
 *
 *	- バイナリ形式で再頒布する場合、頒布物に付属のドキュメント等の
 *		資料に、上記の著作権表示、本条件一覧、および下記免責条項を
 *		含めること。
 *
 *	- 書面による特別の許可なしに、本ソフトウェアから派生した製品の
 *		宣伝または販売促進に、本ソフトウェアの著作権者の名前または
 *		コントリビューターの名前を使用してはならない。
 *
 *	本ソフトウェアは、著作権者およびコントリビューターによって「現
 *	状のまま」提供されており、明示黙示を問わず、商業的な使用可能性、
 *	および特定の目的に対する適合性に関す暗黙の保証も含め、またそれ
 *	に限定されない、いかなる保証もないものとします。著作権者もコン
 *	トリビューターも、事由のいかんを問わず、損害発生の原因いかんを
 *	問わず、かつ責任の根拠が契約であるか厳格責任であるか(過失その
 *	他の)不法行為であるかを問わず、仮にそのような損害が発生する可
 *	能性を知らされていたとしても、本ソフトウェアの使用によって発生
 *	した(代替品または代用サービスの調達、使用の喪失、データの喪失、
 *	利益の喪失、業務の中断も含め、またそれに限定されない)直接損害、
 *	間接損害、偶発的な損害、特別損害、懲罰的損害、または結果損害に
 *	ついて、一切責任を負わないものとします。
 *
 ****************************************************************************/
#include <FK/STLParser.H>
#include <FK/HeapBase.h>

using namespace std;

fk_STLParser::fk_STLParser(void)
{
	return;
}

fk_STLParser::~fk_STLParser()
{
	return;
}

bool fk_STLParser::ReadSTLFile(string argFileName, bool argSolidFlag, double)
{
	FILE					*stlFP;
	vector<fk_Vector>		VecSet;
	vector< vector<int> >	IndexFaceSet;

	if(meshData == (fk_ParserData *)NULL) return false;
	meshData->Init();

	if((stlFP = fopen(argFileName.c_str(), "r")) == (FILE *)NULL) {
		return false;
	}

	if(CheckSTLFile(stlFP) == false) {
		fclose(stlFP);
		return false;
	}

	if(MakeSTLIFSet(stlFP, &VecSet,
					&IndexFaceSet, argSolidFlag) == false) {
		fclose(stlFP);
		return false;
	}
	   
	fclose(stlFP);

	return meshData->MakeMesh(&VecSet, &IndexFaceSet,
							  (vector<int> *)NULL, argSolidFlag);
}

bool fk_STLParser::CheckSTLFile(FILE *argSTLFP)
{
	char	buffer[512];
	string	Line;

	fgets(buffer, 511, argSTLFP);
	Line = buffer;
	StringToLower(&Line);
	if(Line.find("solid") != string::npos) {
		return true;
	}

	return false;
}

bool fk_STLParser::MakeSTLIFSet(FILE *argFP, vector<fk_Vector> *argVecSet,
								vector< vector<int> > *argIFSet,
								bool argSolidFlag)
{
	char				buffer[512];
	string				Line, VecLine;
	bool				LoopFlg, VecFlag;
	vector<int>			VIDSet;
	fk_Vector			tmpVec;
	string::size_type	FindPos;
	unsigned int		size;
	int					index;
	char				*eofStatus;
	fk_HeapBase<fk_Vector>		heap;

	VIDSet.clear();
	LoopFlg = false;
	eofStatus = fgets(buffer, 511, argFP);
	while(feof(argFP) == 0 && eofStatus != (char *)NULL) {
		Line = buffer;
		StringToLower(&Line);

		if(Line.find("outer") != string::npos) {
			if(LoopFlg == true) {
				return false;
			} else {
				LoopFlg = true;
			}
			eofStatus = fgets(buffer, 511, argFP);
			continue;
		}

		if(Line.find("endloop") != string::npos) {
			if(LoopFlg == false) {
				return false;
			} else {
				LoopFlg = false;
				argIFSet->push_back(VIDSet);
				VIDSet.clear();
			}
			eofStatus = fgets(buffer, 511, argFP);
			continue;
		}

		if(LoopFlg == true) {
			if((FindPos = Line.find("vertex")) != string::npos) {
				VecLine = Line.substr(FindPos + 7);
				tmpVec.x = Str2Double(PopWord(&VecLine));
				tmpVec.y = Str2Double(PopWord(&VecLine));
				tmpVec.z = Str2Double(PopWord(&VecLine));

				size = argVecSet->size();
				VecFlag = false;
				if(argSolidFlag == true) {

					index = heap.getID(&tmpVec);
					if(index != int(size)+1) {
						VIDSet.push_back(index);
						VecFlag = true;
					}
				}

				if(VecFlag == false) {
					argVecSet->push_back(tmpVec);
					VIDSet.push_back(size + 1);
				}
			}
		}

		eofStatus = fgets(buffer, 511, argFP);
	}


	if(LoopFlg == true) return false;

	return true;
}

==> FK-2_8_10.tar.bz2: FK-2_8_10/src/STLParser.cxx <==
/****************************************************************************
 *
 *	Copyright (c) 1999-2011, Fine Kernel Project, All rights reserved.
 *
 *	Redistribution and use in source and binary forms,
 *	with or without modification, are permitted provided that the
 *	following conditions are met:
 *
 *		- Redistributions of source code must retain the above
 *			copyright notice, this list of conditions and the
 *			following disclaimer.
 *
 *		- Redistributions in binary form must reproduce the above
 *			copyright notice, this list of conditions and the
 *			following disclaimer in the documentation and/or
 *			other materials provided with the distribution.
 *
 *		- Neither the name of the copyright holders nor the names
 *			of its contributors may be used to endorse or promote
 *			products derived from this software without specific
 *			prior written permission.
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *	"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *	LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *	FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *	COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *	HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *	STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 *	IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *	POSSIBILITY OF SUCH DAMAGE. 
 *
 ****************************************************************************/
/****************************************************************************
 *
 *	Copyright (c) 1999-2011, Fine Kernel Project, All rights reserved.
 *
 *	本ソフトウェアおよびソースコードのライセンスは、基本的に
 *	「修正 BSD ライセンス」に従います。以下にその詳細を記します。
 *
 *	ソースコード形式かバイナリ形式か、変更するかしないかを問わず、
 *	以下の条件を満たす場合に限り、再頒布および使用が許可されます。
 *
 *	- ソースコードを再頒布する場合、上記の著作権表示、本条件一覧、
 *		および下記免責条項を含めること。
 *
 *	- バイナリ形式で再頒布する場合、頒布物に付属のドキュメント等の
 *		資料に、上記の著作権表示、本条件一覧、および下記免責条項を
 *		含めること。
 *
 *	- 書面による特別の許可なしに、本ソフトウェアから派生した製品の
 *		宣伝または販売促進に、本ソフトウェアの著作権者の名前または
 *		コントリビューターの名前を使用してはならない。
 *
 *	本ソフトウェアは、著作権者およびコントリビューターによって「現
 *	状のまま」提供されており、明示黙示を問わず、商業的な使用可能性、
 *	および特定の目的に対する適合性に関す暗黙の保証も含め、またそれ
 *	に限定されない、いかなる保証もないものとします。著作権者もコン
 *	トリビューターも、事由のいかんを問わず、損害発生の原因いかんを
 *	問わず、かつ責任の根拠が契約であるか厳格責任であるか(過失その
 *	他の)不法行為であるかを問わず、仮にそのような損害が発生する可
 *	能性を知らされていたとしても、本ソフトウェアの使用によって発生
 *	した(代替品または代用サービスの調達、使用の喪失、データの喪失、
 *	利益の喪失、業務の中断も含め、またそれに限定されない)直接損害、
 *	間接損害、偶発的な損害、特別損害、懲罰的損害、または結果損害に
 *	ついて、一切責任を負わないものとします。
 *
 ****************************************************************************/

#define FK_DEF_SIZETYPE
#include <FK/STLParser.H>
#include <FK/HeapBase.h>

using namespace std;

fk_STLParser::fk_STLParser(void)
{
	return;
}

fk_STLParser::~fk_STLParser()
{
	return;
}

bool fk_STLParser::ReadSTLFile(string argFileName, bool argSolidFlag, double)
{
	FILE					*stlFP;
	vector<fk_Vector>		VecSet;
	vector< vector<int> >	IndexFaceSet;

	if(meshData == NULL) return false;
	meshData->Init();

	if((stlFP = fopen(argFileName.c_str(), "r")) == NULL) {
		return false;
	}

	if(CheckSTLFile(stlFP) == false) {
		fclose(stlFP);
		return false;
	}

	if(MakeSTLIFSet(stlFP, &VecSet,
					&IndexFaceSet, argSolidFlag) == false) {
		fclose(stlFP);
		return false;
	}
	   
	fclose(stlFP);

	return meshData->MakeMesh(&VecSet, &IndexFaceSet,
							  NULL, argSolidFlag);
}

bool fk_STLParser::CheckSTLFile(FILE *argSTLFP)
{
	char	buffer[512];
	string	Line;

	fgets(buffer, 511, argSTLFP);
	Line = buffer;
	StringToLower(&Line);
	if(Line.find("solid") != string::npos) {
		return true;
	}

	return false;
}

bool fk_STLParser::MakeSTLIFSet(FILE *argFP, vector<fk_Vector> *argVecSet,
								vector< vector<int> > *argIFSet,
								bool argSolidFlag)
{
	char						buffer[512];
	string						Line, VecLine;
	bool						LoopFlg, VecFlag;
	vector<int>					VIDSet;
	fk_Vector					tmpVec;
	string::size_type			FindPos;
	_st							size;
	int							index;
	char						*eofStatus;
	fk_HeapBase<fk_Vector>		heap;

	VIDSet.clear();
	LoopFlg = false;
	eofStatus = fgets(buffer, 511, argFP);
	while(feof(argFP) == 0 && eofStatus != NULL) {
		Line = buffer;
		StringToLower(&Line);

		if(Line.find("outer") != string::npos) {
			if(LoopFlg == true) {
				return false;
			} else {
				LoopFlg = true;
			}
			eofStatus = fgets(buffer, 511, argFP);
			continue;
		}

		if(Line.find("endloop") != string::npos) {
			if(LoopFlg == false) {
				return false;
			} else {
				LoopFlg = false;
				argIFSet->push_back(VIDSet);
				VIDSet.clear();
			}
			eofStatus = fgets(buffer, 511, argFP);
			continue;
		}

		if(LoopFlg == true) {
			if((FindPos = Line.find("vertex")) != string::npos) {
				VecLine = Line.substr(FindPos + 7);
				tmpVec.x = Str2Double(PopWord(&VecLine));
				tmpVec.y = Str2Double(PopWord(&VecLine));
				tmpVec.z = Str2Double(PopWord(&VecLine));

				size = argVecSet->size();
				VecFlag = false;
				if(argSolidFlag == true) {

					index = heap.getID(&tmpVec);
					if(index != static_cast<int>(size)+1) {
						VIDSet.push_back(index);
						VecFlag = true;
					}
				}

				if(VecFlag == false) {
					argVecSet->push_back(tmpVec);
					VIDSet.push_back(static_cast<int>(size) + 1);
				}
			}
		}

		eofStatus = fgets(buffer, 511, argFP);
	}


	if(LoopFlg == true) return false;

	return true;
}
